@@ -63,6 +63,8 @@ appropriate:
 
 from __future__ import print_function
 
+import warnings
+from Bio import BiopythonWarning
 from Bio import MissingPythonDependencyError
 
 try:
@@ -72,9 +74,7 @@ except:
                                        "Bio.motifs.jaspar.db")
 
 from Bio.Alphabet.IUPAC import unambiguous_dna as dna
-
 from Bio.motifs import jaspar, matrix
-from warnings import warn
 
 __docformat__ = "restructuredtext en"
 
@@ -128,16 +128,17 @@ class JASPAR5(object):
 
         Arguments:
 
-            - id - JASPAR matrix ID. This may be a fully specified ID including the
-              version number (e.g. MA0049.2) or just the base ID (e.g. MA0049).
-              If only a base ID is provided, the latest version is returned.
+            - id - JASPAR matrix ID. This may be a fully specified ID including
+              the version number (e.g. MA0049.2) or just the base ID (e.g.
+              MA0049). If only a base ID is provided, the latest version is
+              returned.
 
         Returns:
 
             - A Bio.motifs.jaspar.Motif object
 
-        **NOTE:** The perl TFBS module allows you to specify the type of matrix to
-        return (PFM, PWM, ICM) but matrices are always stored in JASAPR as
+        **NOTE:** The perl TFBS module allows you to specify the type of matrix
+        to return (PFM, PWM, ICM) but matrices are always stored in JASPAR as
         PFMs so this does not really belong here. Once a PFM is fetched the
         pwm() and pssm() methods can be called to return the normalized and
         log-odds matrices.
@@ -147,14 +148,18 @@ class JASPAR5(object):
         # separate stable ID and version number
         (base_id, version) = jaspar.split_jaspar_id(id)
         if not version:
-            # if ID contains no version portion, fetch latest version by default
+            # if ID contains no version portion, fetch the latest version
             version = self._fetch_latest_version(base_id)
 
         # fetch internal JASPAR matrix ID - also a check for validity
-        int_id = self._fetch_internal_id(base_id, version)
+        int_id = None
+        if version:
+            int_id = self._fetch_internal_id(base_id, version)
 
         # fetch JASPAR motif using internal ID
-        motif = self._fetch_motif_by_internal_id(int_id)
+        motif = None
+        if int_id:
+            motif = self._fetch_motif_by_internal_id(int_id)
 
         return motif
 
@@ -194,53 +199,54 @@ class JASPAR5(object):
 
         Arguments::
 
-            Except where obvious, all selection criteria arguments may be specified
-            as a single value or a list of values. Motifs must meet ALL the
-            specified selection criteria to be returned with the precedent
-            exceptions noted below.
+            Except where obvious, all selection criteria arguments may be
+            specified as a single value or a list of values. Motifs must
+            meet ALL the specified selection criteria to be returned with
+            the precedent exceptions noted below.
 
             all         - Takes precedent of all other selection criteria.
                           Every motif is returned. If 'all_versions' is also
                           specified, all versions of every motif are returned,
                           otherwise just the latest version of every motif is
                           returned.
-            matrix_id   - Takes precedence over all other selection criteria except
-                          'all'.  Only motifs with the given JASPAR matrix ID(s)
-                          are returned. A matrix ID may be specified as just a base
-                          ID or full JASPAR IDs including version number. If only a
-                          base ID is provided for specific motif(s), then just the
-                          latest version of those motif(s) are returned unless
+            matrix_id   - Takes precedence over all other selection criteria
+                          except 'all'.  Only motifs with the given JASPAR
+                          matrix ID(s) are returned. A matrix ID may be
+                          specified as just a base ID or full JASPAR IDs
+                          including version number. If only a base ID is
+                          provided for specific motif(s), then just the latest
+                          version of those motif(s) are returned unless
                           'all_versions' is also specified.
             collection  - Only motifs from the specified JASPAR collection(s)
                           are returned. NOTE - if not specified, the collection
-                          defaults to CORE for all other selection criteria except
-                          'all' and 'matrix_id'. To apply the other selection
-                          criteria across all JASPAR collections, explicitly set
-                          collection=None.
+                          defaults to CORE for all other selection criteria
+                          except 'all' and 'matrix_id'. To apply the other
+                          selection criteria across all JASPAR collections,
+                          explicitly set collection=None.
             tf_name     - Only motifs with the given name(s) are returned.
             tf_class    - Only motifs of the given TF class(es) are returned.
             tf_family   - Only motifs from the given TF families are returned.
-            tax_group   - Only motifs belonging to the given taxonomic supergroups
-                          are returned (e.g. 'vertebrates', 'insects', 'nematodes'
-                          etc.)
-            species     - Only motifs derived from the given species are returned.
-                          Species are specified as taxonomy IDs.
+            tax_group   - Only motifs belonging to the given taxonomic
+                          supergroups are returned (e.g. 'vertebrates',
+                          'insects', 'nematodes' etc.)
+            species     - Only motifs derived from the given species are
+                          returned.  Species are specified as taxonomy IDs.
             data_type   - Only motifs generated with the given data type (e.g.
-                          ('ChIP-seq', 'PBM', 'SELEX' etc.) are returned. NOTE -
-                          must match exactly as stored in the database.
+                          ('ChIP-seq', 'PBM', 'SELEX' etc.) are returned.
+                          NOTE - must match exactly as stored in the database.
             pazar_id    - Only motifs with the given PAZAR TF ID are returned.
             medline     - Only motifs with the given medline (PubmMed IDs) are 
                           returned.
             min_ic      - Only motifs whose profile matrices have at least this
                           information content (specificty) are returned.
-            min_length  - Only motifs whose profiles are of at least this length
-                          are returned.
+            min_length  - Only motifs whose profiles are of at least this
+                          length are returned.
             min_sites   - Only motifs compiled from at least these many binding
                           sites are returned.
             all_versions- Unless specified, just the latest version of motifs
-                          determined by the other selection criteria are returned
-                          otherwise all versions of the selected motifs are
-                          returned.
+                          determined by the other selection criteria are
+                          returned. Otherwise all versions of the selected
+                          motifs are returned.
 
         Returns:
 
@@ -308,12 +314,17 @@ class JASPAR5(object):
 
         """
 
-        sql = "select VERSION from MATRIX where BASE_id = '%s' order by VERSION desc limit 1" % base_id
-
         cur = self.dbh.cursor()
-        cur.execute(sql)
+        cur.execute("""select VERSION from MATRIX where BASE_id = %s
+                       order by VERSION desc limit 1""", (base_id,))
 
-        latest = cur.fetchone()[0]
+        row = cur.fetchone()
+
+        latest = None
+        if row:
+            latest = row[0]
+        else:
+            warnings.warn("Failed to fetch latest version number for JASPAR motif with base ID '{0}'. No JASPAR motif with this base ID appears to exist in the database.".format(base_id), BiopythonWarning)
 
         return latest
 
@@ -324,24 +335,34 @@ class JASPAR5(object):
 
         """
 
-        sql = "select id from MATRIX where BASE_id = '%s' and VERSION = '%s'" % (base_id, version)
-
         cur = self.dbh.cursor()
-        cur.execute(sql)
+        cur.execute("""select id from MATRIX where BASE_id = %s
+                       and VERSION = %s""", (base_id, version))
 
-        int_id = cur.fetchone()[0]
+        row = cur.fetchone()
+
+        int_id = None
+        if row:
+            int_id = row[0]
+        else:
+            warnings.warn("Failed to fetch internal database ID for JASPAR motif with matrix ID '{0}.{1}'. No JASPAR motif with this matrix ID appears to exist.".format(base_id, version), BiopythonWarning)
 
         return int_id
 
     def _fetch_motif_by_internal_id(self, int_id):
         # fetch basic motif information
-        sql = "select BASE_ID, VERSION, COLLECTION, NAME from MATRIX where id = %d" % int_id
-
         cur = self.dbh.cursor()
-        cur.execute(sql)
+        cur.execute("""select BASE_ID, VERSION, COLLECTION, NAME from MATRIX
+                       where id = %s""", (int_id,))
 
         row = cur.fetchone()
 
+        # This should never happen as it is an internal method. If it does
+        # we should probably raise an exception
+        if not row:
+            warnings.warn("Could not fetch JASPAR motif with internal ID = {0}".format(int_id), BiopythonWarning)
+            return None
+
         base_id = row[0]
         version = row[1]
         collection = row[2]
@@ -358,28 +379,35 @@ class JASPAR5(object):
         )
 
         # fetch species
-        sql = "select TAX_ID from MATRIX_SPECIES where id = %d" % int_id
-        cur.execute(sql)
+        cur.execute("""select TAX_ID from MATRIX_SPECIES
+                       where id = %s""", (int_id,))
         tax_ids = []
         rows = cur.fetchall()
         for row in rows:
             tax_ids.append(row[0])
 
+        # Many JASPAR motifs (especially those not in the CORE collection)
+        # do not have taxonomy IDs. So this warning would get annoying.
+        #if not tax_ids:
+        #    warnings.warn("Could not fetch any taxonomy IDs for JASPAR motif {0}".format(motif.matrix_id), BiopythonWarning)
+
         motif.species = tax_ids
 
         # fetch protein accession numbers
-        sql = "select ACC FROM MATRIX_PROTEIN where id = %d" % int_id
-        cur.execute(sql)
+        cur.execute("select ACC FROM MATRIX_PROTEIN where id = %s", (int_id,))
         accs = []
         rows = cur.fetchall()
         for row in rows:
             accs.append(row[0])
 
+        # Similarly as for taxonomy IDs, it would get annoying to print
+        # warnings for JASPAR motifs which do not have accession numbers.
+
         motif.acc = accs
 
         # fetch remaining annotation as tags from the ANNOTATION table
-        sql = "select TAG, VAL from MATRIX_ANNOTATION where id = %d" % int_id
-        cur.execute(sql)
+        cur.execute("""select TAG, VAL from MATRIX_ANNOTATION
+                       where id = %s""", (int_id,))
         rows = cur.fetchall()
         for row in rows:
             attr = row[0]
@@ -420,7 +448,8 @@ class JASPAR5(object):
         for base in dna.letters:
             base_counts = []
 
-            cur.execute("select val from MATRIX_DATA where ID = %s and row = %s order by col", (int_id, base))
+            cur.execute("""select val from MATRIX_DATA where ID = %s
+                           and row = %s order by col""", (int_id, base))
 
             rows = cur.fetchall()
             for row in rows:
@@ -499,7 +528,7 @@ class JASPAR5(object):
                     # ignore vesion here, this is a stupidity filter
                     (base_id, version) = jaspar.split_jaspar_id(id)
                     cur.execute(
-                        "select ID from MATRIX where BASE_ID = %s", base_id
+                        "select ID from MATRIX where BASE_ID = %s", (base_id,)
                     )
 
                     rows = cur.fetchall()
@@ -513,7 +542,9 @@ class JASPAR5(object):
                     if not version:
                         version = self._fetch_latest_version(base_id)
 
-                    int_id = self._fetch_internal_id(base_id, version)
+                    int_id = None
+                    if version:
+                        int_id = self._fetch_internal_id(base_id, version)
 
                     if int_id:
                         int_ids.append(int_id)
@@ -714,19 +745,24 @@ class JASPAR5(object):
                     int_ids.append(id)
 
         if len(int_ids) < 1:
-            warn("Warning: Zero motifs returned with current select critera")
+            warnings.warn("Zero motifs returned with current select critera", BiopythonWarning)
 
         return int_ids
 
     def _is_latest_version(self, int_id):
         """
-        Does this internal ID represened the latest version of the JASPAR
+        Does this internal ID represent the latest version of the JASPAR
         matrix (collapse on base ids)
 
         """
         cur = self.dbh.cursor()
 
-        cur.execute("select count(*) from MATRIX where BASE_ID = (select BASE_ID from MATRIX where ID = %s) and VERSION > (select VERSION from MATRIX where ID = %s)", (int_id, int_id))
+        cur.execute(
+            """select count(*) from MATRIX
+               where BASE_ID = (select BASE_ID from MATRIX where ID = %s)
+               and VERSION > (select VERSION from MATRIX where ID = %s)""",
+               (int_id, int_id)
+        )
 
         row = cur.fetchone()
 
